@@ -1,11 +1,18 @@
+"""Module providing a markdown drawer widget for the numerous library."""
+
+import textwrap
+
 import anywidget
 import traitlets
-from ._config import get_widget_paths
+
+from .config import get_widget_paths
+
 
 # Get environment-appropriate paths
 ESM, CSS = get_widget_paths("MarkdownDrawerWidget")
 
-class MarkdownDrawer(anywidget.AnyWidget):
+
+class MarkdownDrawer(anywidget.AnyWidget):  # type: ignore[misc]
     """
     A collapsible drawer widget that displays markdown content.
 
@@ -13,7 +20,9 @@ class MarkdownDrawer(anywidget.AnyWidget):
         title: The title shown in the drawer header
         content: The markdown content to display
         open: Whether the drawer starts open (default: False)
+
     """
+
     # Define traitlets for the widget properties
     title = traitlets.Unicode().tag(sync=True)
     content = traitlets.Unicode().tag(sync=True)
@@ -27,10 +36,12 @@ class MarkdownDrawer(anywidget.AnyWidget):
         self,
         title: str,
         content: str,
-        open: bool = False,
-    ):
+        is_open: bool = False,
+    ) -> None:
+        # Dedent the content to remove unnecessary indentation
+        dedented_content = textwrap.dedent(content)
         super().__init__(
             title=title,
-            content=content,
-            is_open=open,
+            content=dedented_content,
+            is_open=is_open,
         )
